﻿using Infragistics.Win.UltraWinGrid;
using Infragistics.Win.UltraWinToolbars;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace _143068
{
    public class MyGrid:UltraGrid
    {

        public delegate void BeforeContextMenuDropDownHandler(object sender, ContextMenuArgs e);
        public event BeforeContextMenuDropDownHandler BeforeContextMenuDropDown;

        public delegate void ExtendToolListkHandler(object sender, ToolClickEventArgs e);
        public event ExtendToolListkHandler ExtendToolListClick;


        UltraToolbarsManager toolbarsManager;
        PopupMenuTool popupMenuTool1 = new PopupMenuTool("PopupMenuTool1");
        ButtonTool buttonTool1 = new ButtonTool("ButtonTool1");
        ButtonTool buttonTool2 = new ButtonTool("ButtonTool2");
        ButtonTool buttonTool3 = new ButtonTool("ButtonTool3");
        ButtonTool buttonTool4 = new ButtonTool("ButtonTool4");
        ButtonTool buttonTool5 = new ButtonTool("ButtonTool5");

        ExtendList<ToolBase> extendToolList;

        public ExtendList<ToolBase> ExtendToolList
        {
            get { return extendToolList; }
        } 

        public MyGrid()
            : base()
        {
            extendToolList = new ExtendList<ToolBase>();
            toolbarsManager = new UltraToolbarsManager();
            //this.ultraToolbarsManager1 = new Infragistics.Win.UltraWinToolbars.UltraToolbarsManager(this.components);
            
            buttonTool1.SharedPropsInternal.Caption = "ButtonTool1";
            buttonTool2.SharedPropsInternal.Caption = "ButtonTool2";
            buttonTool3.SharedPropsInternal.Caption = "ButtonTool3";
            buttonTool4.SharedPropsInternal.Caption = "ButtonTool4";
            buttonTool5.SharedPropsInternal.Caption = "ButtonTool5";
            
            this.toolbarsManager.Tools.AddRange(new Infragistics.Win.UltraWinToolbars.ToolBase[] {
            popupMenuTool1,
            buttonTool1,
            buttonTool2,
            buttonTool3,
            buttonTool4,
            buttonTool5});

            popupMenuTool1.Tools.AddToolRange(new string[] {
            buttonTool1.Key,
            buttonTool2.Key,
            buttonTool3.Key,
            buttonTool4.Key,
            buttonTool5.Key});

            this.toolbarsManager.SetContextMenuUltra(this, "PopupMenuTool1");

            extendToolList.OnBeforeAddItem += extendToolList_OnBeforeAddItem;
            extendToolList.OnBeforeRemoveItem += extendToolList_OnBeforeRemoveItem;
            extendToolList.OnBeforeClearItem += extendToolList_OnBeforeClearItem;
            toolbarsManager.BeforeToolDropdown += toolbarsManager_BeforeToolDropdown;
            toolbarsManager.ToolClick += toolbarsManager_ToolClick;

        }

        void toolbarsManager_ToolClick(object sender, ToolClickEventArgs e)
        {
            if (ExtendToolListClick != null && extendToolList.FirstOrDefault(t => t.Key == e.Tool.Key) != null)
            {
                ExtendToolListClick.Invoke(sender, e);
            } 
 
        }

        void extendToolList_OnBeforeClearItem(object sender, CancelEventArgs e)
        {

            try
            {
                foreach (ToolBase tool in extendToolList)
                {
                    popupMenuTool1.Tools.Remove(tool);
                    this.toolbarsManager.Tools.Remove(tool);
                }
            }
            catch (Exception ex)
            {
                e.Cancel = true;
                throw ex;
            }
        }

        void extendToolList_OnBeforeRemoveItem(object sender, ListArgs<ToolBase> e)
        {
            try
            {
                popupMenuTool1.Tools.Remove(e.Item);
                this.toolbarsManager.Tools.Remove(e.Item); 
            }
            catch (Exception ex)
            {
                e.Cancel = true;
                throw ex;
            }
        }

        void extendToolList_OnBeforeAddItem(object sender, ListArgs<ToolBase> e)
        {
            try
            {
                InstanceProps props = e.Item.InstanceProps;
                int index = this.toolbarsManager.Tools.Add(e.Item);
               
                ToolBase tool = popupMenuTool1.Tools.AddTool(e.Item.Key);
                tool.InstanceProps.IsFirstInGroup = props.IsFirstInGroup;//.IsFirstInGroup = e.Item.InstanceProps.IsFirstInGroup;
            }
            catch (Exception ex)
            {
                e.Cancel = true;
                throw ex;
            }
        }

        void toolbarsManager_BeforeToolDropdown(object sender, BeforeToolDropdownEventArgs e)
        {
            ContextMenuArgs arg = new ContextMenuArgs(ref extendToolList);

            if (BeforeContextMenuDropDown != null)
                BeforeContextMenuDropDown.Invoke(this, arg);
            e.Cancel = arg.Cancel;
            
        }

        protected override void Dispose(bool disposing)
        {
            extendToolList.OnBeforeAddItem -= extendToolList_OnBeforeAddItem;
            extendToolList.OnBeforeRemoveItem -= extendToolList_OnBeforeRemoveItem;
            extendToolList.OnBeforeClearItem -= extendToolList_OnBeforeClearItem;
            toolbarsManager.BeforeToolDropdown -= toolbarsManager_BeforeToolDropdown;
            toolbarsManager.ToolClick -= toolbarsManager_ToolClick;
            extendToolList.Dispose();
            toolbarsManager.Dispose();
            base.Dispose(disposing);
        }
    }

    public class ContextMenuArgs : CancelEventArgs
    {
        ExtendList<ToolBase> extendToolList;

        public ExtendList<ToolBase> ExtendToolList
        {
            get { return extendToolList; }
        }

        public ContextMenuArgs(ref ExtendList<ToolBase> extendToolList)
        {
            this.extendToolList = extendToolList;
        }
    }

    public class ExtendList<T> : IList<T>, IDisposable
    {

        public delegate void BeforeRemoveHandler(object sender, ListArgs<T> e);
        public event BeforeRemoveHandler OnBeforeRemoveItem;

        public delegate void BeforeAddHandler(object sender, ListArgs<T> e);
        public event BeforeAddHandler OnBeforeAddItem;

        public delegate void BeforeClearHandler(object sender, CancelEventArgs e);
        public event BeforeClearHandler OnBeforeClearItem;

        List<T> innerList;

        public ExtendList()
        {
            innerList = new List<T>();
        }

        public int IndexOf(T item)
        {
            return innerList.IndexOf(item);
        }

        public void Insert(int index, T item)
        {
            if (!FireAddEvent(item))
                innerList.Insert(index, item);
        }

        public void RemoveAt(int index)
        {
            if (!FireRemoveEvent(innerList[index]))
                innerList.RemoveAt(index);
        }

        public T this[int index]
        {
            get
            {
                return innerList[index];
            }
            set
            {
                innerList[index] = value;
            }
        }

        public void Add(T item)
        {
            if (!FireAddEvent(item))
                innerList.Add( item);
        }

        public void Clear()
        {
            if (!FireClearEvent())
                innerList.Clear();
        }

        public bool Contains(T item)
        {
            return innerList.Contains(item);
        }

        public void CopyTo(T[] array, int arrayIndex)
        {
            innerList.CopyTo(array, arrayIndex);
        }

        public int Count
        {
            get { return innerList.Count(); }
        }

        public bool IsReadOnly
        {
            get { return false; }
        }

        public bool Remove(T item)
        {
            bool result = false;
            if (!FireRemoveEvent(item))
                result = innerList.Remove(item);
            return result;
        }

        public IEnumerator<T> GetEnumerator()
        {
            return innerList.GetEnumerator();
        }

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return innerList.GetEnumerator();
        }

        private bool FireAddEvent(T item)
        {
            ListArgs<T> result = new ListArgs<T>(ref item);
            if (OnBeforeAddItem != null)
                OnBeforeAddItem.Invoke(this, result);

            return result.Cancel;
        }

        private bool FireRemoveEvent(T item)
        {
            ListArgs<T> result = new ListArgs<T>(ref item);
            if (OnBeforeRemoveItem != null)
                OnBeforeRemoveItem.Invoke(this, result);

            return result.Cancel;
        }

        private bool FireClearEvent()
        {
            CancelEventArgs result = new CancelEventArgs();
            if (OnBeforeClearItem != null)
                OnBeforeClearItem.Invoke(this, result);

            return result.Cancel;
        }

        public void Dispose()
        {
            innerList.Clear();
        }
    }

    public class ListArgs<T> : CancelEventArgs
    {
        T item;

        public T Item
        {
            get { return item; }
            //set { item = value; }
        }

        public ListArgs(ref T item)
        {
            this.item = item;
        }
    }
}
